@@ -3,16 +3,20 @@ use warnings;
 use strict;
 use Carp;
 use utf8;
+
 require Exporter;
+
 our @ISA = qw(Exporter);
 our @EXPORT_OK = qw/
     copy_css
     tidy_css
 /;
+
 our %EXPORT_TAGS = (
     all => \@EXPORT_OK,
 );
-our $VERSION = '0.02';
+
+our $VERSION = '0.03';
 
 use C::Tokenize '$comment_re';
 use File::Slurper qw!read_text write_text!;
@@ -58,6 +62,9 @@ sub tidy_css
     my ($text, %options) = @_;
 
     my $decomment = get_option (\%options, 'decomment');
+    #my $verbose = 
+    # Discard this at the moment, we have no verbose output yet.
+    get_option (\%options, 'verbose');
     check_options (\%options);
 
     # Store for comments during processing. They are then restored.
@@ -68,6 +75,10 @@ sub tidy_css
     else {
 	($text, $comments) = strip_comments ($text);
     }
+    $text =~ s!(\{|\}|;)(\s*\S)!$1\n$2!g;
+    $text =~ s!(\S\s*)(\})!$1\n$2!g;
+    $text =~ s!(\S)(\{)!$1 $2!g;
+
     my @lines = split /\n/, $text;
 
     my @tidy;
@@ -130,10 +141,18 @@ sub tidy_css
     $out =~ s/\n+/\n/g;
     # Add a blank after }
     $out =~ s/^(\s*\})/$1\n/gsm;
+
+    $out =~ s/^\}\n(\S)/\}\n\n$1/gsm;
     # Remove a blank line before }. This also tidies up the
     # aftereffects of the above regex, which puts too many blank
     # lines.
     $out =~ s/\n\n(\s*\})/\n$1/g;
+
+    # Add a semicolon after the final CSS instruction if there is not
+    # one.
+
+    $out =~ s!([^\};])\s*(\n\s*\})!$1;$2!g;
+
     if (! $decomment) {
 	$out = restore_comments ($out, $comments);
     }
@@ -151,17 +170,37 @@ sub rm_comments
     return $text;
 }
 
+my $string_re = qr!"(\\"|[^"])*"!;
+
 # Strip the comments out in such a way that they can be restored.
 
 sub strip_comments
 {
     my ($text) = @_;
+
+    # Remove and store all strings so that "http://example.com"
+    # doesn't get turned into a comment.
+
+    my @strings;
+    my $s = 0;
+    while ($text =~ s!($string_re)!\@\@ string_#$s \@\@!sm) {
+	$s++;
+	push @strings, $1;
+    }
+
+    # Remove and store comments.
+
     my @comments;
     my $n = 0;
     while ($text =~ s!($comment_re)!/\@ css_tidy_#$n \@/!sm) {
 	$n++;
 	push @comments, $1;
     }
+
+    # Restore the strings.
+
+    $text =~ s!\@\@ string_#([0-9]+) \@\@!$strings[$1]!g;
+
     return ($text, \@comments);
 }
 
